@@ -61,15 +61,32 @@
 
 static void
 record_set_username (UTMP       *u,
-                     const char *username)
+                     const char *user_name)
 {
+        const char *username;
+
+        /*
+         * It is possible that PAM failed before it mapped the user
+         * input into a valid username, so we fallback to try using
+         * "(unknown)"
+         */
+        if (user_name != NULL) {
+                username = user_name;
+        } else {
+                username = "(unknown)";
+        }
+
 #if defined(HAVE_UT_UT_USER)
-        memccpy (u->ut_user, username, '\0', sizeof (u->ut_user));
+        strncpy (u->ut_user,
+                 username,
+                 sizeof (u->ut_user));
         g_debug ("using ut_user %.*s",
                  (int) sizeof (u->ut_user),
                  u->ut_user);
 #elif defined(HAVE_UT_UT_NAME)
-        memccpy (u->ut_name, username, '\0', sizeof (u->ut_name));
+        strncpy (u->ut_name,
+                 username,
+                 sizeof (u->ut_name));
         g_debug ("using ut_name %.*s",
                  (int) sizeof (u->ut_name),
                  u->ut_name);
@@ -109,6 +126,16 @@ record_set_pid (UTMP *u,
 #endif
 }
 
+static void
+record_set_id (UTMP       *u,
+               const char *id)
+{
+#if defined(HAVE_UT_UT_ID)
+        strncpy (u->ut_id, id, sizeof (u->ut_id));
+        g_debug ("using ut_id %.*s", (int) sizeof (u->ut_id), u->ut_id);
+#endif
+}
+
 static void
 record_set_host (UTMP       *u,
                  const char *x11_display_name,
@@ -132,12 +159,13 @@ record_set_host (UTMP       *u,
         }
 
         if (hostname != NULL) {
-                memccpy (u->ut_host, hostname, '\0', sizeof (u->ut_host));
+                strncpy (u->ut_host, hostname, sizeof (u->ut_host));
                 g_debug ("using ut_host %.*s", (int) sizeof (u->ut_host), u->ut_host);
+                g_free (hostname);
+
 #ifdef HAVE_UT_UT_SYSLEN
                 u->ut_syslen = MIN (strlen (hostname), sizeof (u->ut_host));
 #endif
-                g_free (hostname);
         }
 #endif
 }
@@ -154,14 +182,13 @@ record_set_line (UTMP       *u,
          */
         if (display_device != NULL
             && g_str_has_prefix (display_device, "/dev/")) {
-                memccpy (u->ut_line,
+                strncpy (u->ut_line,
                          display_device + strlen ("/dev/"),
-                         '\0',
                          sizeof (u->ut_line));
-        } else if (x11_display_name != NULL) {
-                memccpy (u->ut_line,
+        } else if (x11_display_name != NULL
+                   && g_str_has_prefix (x11_display_name, ":")) {
+                strncpy (u->ut_line,
                          x11_display_name,
-                         '\0',
                          sizeof (u->ut_line));
         }
 
@@ -176,9 +203,7 @@ gdm_session_record_login (GPid                  session_pid,
                           const char           *display_device)
 {
         UTMP        session_record = { 0 };
-
-        if (x11_display_name == NULL)
-                x11_display_name = display_device;
+        UTMP       *u;
 
         record_set_username (&session_record, user_name);
 
@@ -191,6 +216,9 @@ gdm_session_record_login (GPid                  session_pid,
 
         record_set_timestamp (&session_record);
         record_set_pid (&session_record, session_pid);
+
+        /* Set ut_id to the $DISPLAY value */
+        record_set_id (&session_record, x11_display_name);
         record_set_host (&session_record, x11_display_name, host_name);
         record_set_line (&session_record, display_device, x11_display_name);
 
@@ -208,12 +236,31 @@ gdm_session_record_login (GPid                  session_pid,
 #endif
 #endif
 
-        /* Handle utmp */
+        /*
+         * Handle utmp
+         * Update if entry already exists
+         */
 #if defined(HAVE_GETUTXENT)
-        g_debug ("Adding or updating utmp record for login");
-        setutxent();
-        pututxline (&session_record);
-        endutxent();
+        setutxent ();
+
+        while ((u = getutxent ()) != NULL) {
+                if (u->ut_type == USER_PROCESS &&
+                    (session_record.ut_line != NULL &&
+                     (strncmp (u->ut_line, session_record.ut_line,
+                               sizeof (u->ut_line)) == 0 ||
+                      u->ut_pid == session_record.ut_pid))) {
+                        g_debug ("Updating existing utmp record");
+                        pututxline (&session_record);
+                        break;
+                }
+        }
+        endutxent ();
+
+        /* Add new entry if update did not work */
+        if (u == NULL) {
+                g_debug ("Adding new utmp record");
+                pututxline (&session_record);
+        }
 #elif defined(HAVE_LOGIN)
 	login (&session_record);
 #endif
@@ -227,9 +274,7 @@ gdm_session_record_logout (GPid                  session_pid,
                            const char           *display_device)
 {
         UTMP        session_record = { 0 };
-
-        if (x11_display_name == NULL)
-                x11_display_name = display_device;
+        UTMP       *u;
 
         g_debug ("Writing logout record");
 
@@ -240,9 +285,12 @@ gdm_session_record_logout (GPid                  session_pid,
 
         record_set_timestamp (&session_record);
         record_set_pid (&session_record, session_pid);
+        /* Set ut_id to the $DISPLAY value */
+        record_set_id (&session_record, x11_display_name);
         record_set_host (&session_record, x11_display_name, host_name);
         record_set_line (&session_record, display_device, x11_display_name);
 
+
         /* Handle wtmp */
         g_debug ("Writing wtmp logout record to " GDM_NEW_SESSION_RECORDS_FILE);
 #if defined(HAVE_UPDWTMPX)
@@ -255,10 +303,36 @@ gdm_session_record_logout (GPid                  session_pid,
 
         /* Handle utmp */
 #if defined(HAVE_GETUTXENT)
-        g_debug ("Adding or updating utmp record for logout");
-        setutxent();
-        pututxline (&session_record);
-        endutxent();
+        setutxent ();
+
+        while ((u = getutxent ()) != NULL &&
+               (u = getutxid (&session_record)) != NULL) {
+
+                g_debug ("Removing utmp record");
+                if (u->ut_pid == session_pid &&
+                    u->ut_type == DEAD_PROCESS) {
+                        /* Already done */
+                        break;
+                }
+
+                u->ut_type = DEAD_PROCESS;
+#if defined(HAVE_UT_UT_TV)
+                u->ut_tv.tv_sec  = session_record.ut_tv.tv_sec;
+                u->ut_tv.tv_usec = session_record.ut_tv.tv_usec;
+#elif defined(HAVE_UT_UT_TIME)
+                u->ut_time = session_record.ut_time;
+#endif
+#ifdef HAVE_UT_UT_EXIT_E_TERMINATION
+                u->ut_exit.e_termination = 0;
+                u->ut_exit.e_exit = 0;
+#endif
+
+                pututxline (u);
+
+                break;
+        }
+
+        endutxent ();
 #elif defined(HAVE_LOGOUT)
         logout (session_record.ut_line);
 #endif
@@ -273,9 +347,6 @@ gdm_session_record_failed (GPid                  session_pid,
 {
         UTMP        session_record = { 0 };
 
-        if (x11_display_name == NULL)
-                x11_display_name = display_device;
-
         record_set_username (&session_record, user_name);
 
         g_debug ("Writing failed session attempt record");
@@ -287,14 +358,14 @@ gdm_session_record_failed (GPid                  session_pid,
 
         record_set_timestamp (&session_record);
         record_set_pid (&session_record, session_pid);
+        /* Set ut_id to the $DISPLAY value */
+        record_set_id (&session_record, x11_display_name);
         record_set_host (&session_record, x11_display_name, host_name);
         record_set_line (&session_record, display_device, x11_display_name);
 
-#if defined(HAVE_UPDWTMPX) || defined(HAVE_UPDWTMP)
         /* Handle btmp */
         g_debug ("Writing btmp failed session attempt record to "
                  GDM_BAD_SESSION_RECORDS_FILE);
-#endif
 
 #if defined(HAVE_UPDWTMPX)
         updwtmpx (GDM_BAD_SESSION_RECORDS_FILE, &session_record);
