@@ -65,6 +65,7 @@
 #define DEFAULT_PING_INTERVAL 15
 
 #define INITIAL_SETUP_USERNAME "gnome-initial-setup"
+#define INITIAL_SETUP_TRIGGER_FILE LOCALSTATEDIR "/lib/gdm/run-initial-setup"
 
 struct GdmSimpleSlavePrivate
 {
@@ -115,6 +116,8 @@ static void     gdm_simple_slave_open_reauthentication_channel (GdmSlave
                                                                 gpointer              user_data,
                                                                 GCancellable         *cancellable);
 
+static gboolean wants_initial_setup (GdmSimpleSlave *slave);
+static void destroy_initial_setup_user (GdmSimpleSlave *slave);
 G_DEFINE_TYPE (GdmSimpleSlave, gdm_simple_slave, GDM_TYPE_SLAVE)
 
 static void create_new_session (GdmSimpleSlave  *slave);
@@ -308,17 +311,18 @@ gdm_simple_slave_start_session_when_ready (GdmSimpleSlave *slave,
 }
 
 static gboolean
-try_migrate_session (GdmSimpleSlave  *slave)
+switch_to_and_unlock_session (GdmSimpleSlave  *slave,
+                              gboolean         fail_if_already_switched)
 {
         char    *username;
         gboolean res;
 
-        g_debug ("GdmSimpleSlave: trying to migrate session");
-
         username = gdm_session_get_username (slave->priv->session);
 
+        g_debug ("GdmSimpleSlave: trying to switch to session for user %s", username);
+
         /* try to switch to an existing session */
-        res = gdm_slave_switch_to_user_session (GDM_SLAVE (slave), username);
+        res = gdm_slave_switch_to_user_session (GDM_SLAVE (slave), username, fail_if_already_switched);
         g_free (username);
 
         return res;
@@ -389,11 +393,18 @@ static gboolean
 start_session_timeout (GdmSimpleSlave  *slave)
 {
         gboolean migrated;
-
+        gboolean fail_if_already_switched = TRUE;
 
         g_debug ("GdmSimpleSlave: accredited");
 
-        migrated = try_migrate_session (slave);
+        /* If there's already a session running, jump to it.
+         * If the only session running is the one we just opened,
+         * start a session on it.
+         *
+         * We assume we're in the former case if we need to switch
+         * VTs, and we assume we're in the latter case if we don't.
+         */
+        migrated = switch_to_and_unlock_session (slave, fail_if_already_switched);
         g_debug ("GdmSimpleSlave: migrated: %d", migrated);
         if (migrated) {
                 /* We don't stop the slave here because
@@ -435,7 +446,14 @@ on_session_reauthenticated (GdmSession       *session,
                             const char       *service_name,
                             GdmSimpleSlave   *slave)
 {
-        try_migrate_session (slave);
+        gboolean fail_if_already_switched = FALSE;
+
+        /* There should already be a session running, so jump to it's
+         * VT. In the event we're already on the right VT, (i.e. user
+         * used an unlock screen instead of a user switched login screen),
+         * then silently succeed and unlock the session.
+         */
+        switch_to_and_unlock_session (slave, fail_if_already_switched);
 }
 
 static void
@@ -842,6 +860,9 @@ on_greeter_environment_session_stopped (GdmLaunchEnvironment *greeter_environmen
         if (slave->priv->start_session_service_name == NULL) {
                 gdm_slave_stop (GDM_SLAVE (slave));
         } else {
+                if (wants_initial_setup (slave)) {
+                        destroy_initial_setup_user (slave);
+                }
                 start_session (slave);
         }
 
@@ -1118,7 +1139,7 @@ start_greeter (GdmSimpleSlave *slave)
         start_launch_environment (slave, GDM_USERNAME, NULL);
 }
 
-#define RULES_DIR LOCALSTATEDIR "/lib/polkit-1/localauthority/10-vendor.d/"
+#define RULES_DIR DATADIR "/polkit-1/rules.d/"
 #define RULES_FILE "20-gnome-initial-setup.rules"
 
 static const gboolean
@@ -1147,12 +1168,14 @@ create_initial_setup_user (GdmSimpleSlave *slave)
                         ret = FALSE;
                         goto out;
                 }
+
+                g_clear_error (&error);
         } else {
                 g_object_unref (user);
         }
 
         /* Now, make sure the PolicyKit policy is in place */
-        src_file = g_file_new_for_path (DATADIR "/gnome-initial-setup" RULES_FILE);
+        src_file = g_file_new_for_path (DATADIR "/gnome-initial-setup/" RULES_FILE);
         dest_file = g_file_new_for_path (RULES_DIR RULES_FILE);
 
         if (!g_file_copy (src_file,
@@ -1195,12 +1218,17 @@ destroy_initial_setup_user (GdmSimpleSlave *slave)
 
         error = NULL;
         user = act_user_manager_get_user (act, INITIAL_SETUP_USERNAME);
-        if (!act_user_manager_delete_user (act, user, TRUE, &error)) {
-                g_warning ("Failed to delete user '%s': %s", INITIAL_SETUP_USERNAME, error->message);
-                g_error_free (error);
+        if (user != NULL) {
+                if (!act_user_manager_delete_user (act, user, TRUE, &error)) {
+                        g_warning ("Failed to delete user '%s': %s", INITIAL_SETUP_USERNAME, error->message);
+                        g_error_free (error);
+                }
+                g_object_unref (user);
         }
 
-        g_object_unref (user);
+        if (g_remove (INITIAL_SETUP_TRIGGER_FILE) < 0) {
+                g_warning ("Failed to remove '%s': %s", INITIAL_SETUP_TRIGGER_FILE, g_strerror (errno));
+        }
 }
 
 static void
@@ -1208,7 +1236,6 @@ start_initial_setup (GdmSimpleSlave *slave)
 {
         create_initial_setup_user (slave);
         start_launch_environment (slave, INITIAL_SETUP_USERNAME, "gnome-initial-setup");
-        destroy_initial_setup_user (slave);
 }
 
 static gboolean
@@ -1222,8 +1249,6 @@ wants_autologin (GdmSimpleSlave *slave)
         return enabled && delay == 0;
 }
 
-#define INITIAL_SETUP_TRIGGER_FILE LOCALSTATEDIR "/lib/gdm/run-initial-setup"
-
 static gboolean
 wants_initial_setup (GdmSimpleSlave *slave)
 {
