@@ -545,11 +545,15 @@ gdm_slave_setup_xhost_auth (XHostAddress *host_entries, XServerInterpretedAddres
         si_entries[0].typelength  = strlen ("localuser");
         si_entries[1].type        = "localuser";
         si_entries[1].typelength  = strlen ("localuser");
+        si_entries[2].type        = "localuser";
+        si_entries[2].typelength  = strlen ("localuser");
 
         si_entries[0].value       = "root";
         si_entries[0].valuelength = strlen ("root");
         si_entries[1].value       = GDM_USERNAME;
         si_entries[1].valuelength = strlen (GDM_USERNAME);
+        si_entries[2].value       = "gnome-initial-setup";
+        si_entries[2].valuelength = strlen ("gnome-initial-setup");
 
         host_entries[0].family    = FamilyServerInterpreted;
         host_entries[0].address   = (char *) &si_entries[0];
@@ -557,6 +561,9 @@ gdm_slave_setup_xhost_auth (XHostAddress *host_entries, XServerInterpretedAddres
         host_entries[1].family    = FamilyServerInterpreted;
         host_entries[1].address   = (char *) &si_entries[1];
         host_entries[1].length    = sizeof (XServerInterpretedAddress);
+        host_entries[2].family    = FamilyServerInterpreted;
+        host_entries[2].address   = (char *) &si_entries[2];
+        host_entries[2].length    = sizeof (XServerInterpretedAddress);
 }
 
 static void
@@ -672,8 +679,8 @@ gdm_slave_connect_to_x11_display (GdmSlave *slave)
                 g_warning ("Unable to connect to display %s", slave->priv->display_name);
                 ret = FALSE;
         } else if (slave->priv->display_is_local) {
-                XServerInterpretedAddress si_entries[2];
-                XHostAddress              host_entries[2];
+                XServerInterpretedAddress si_entries[3];
+                XHostAddress              host_entries[3];
 
                 g_debug ("GdmSlave: Connected to display %s", slave->priv->display_name);
                 ret = TRUE;
@@ -909,8 +916,8 @@ gdm_slave_add_user_authorization (GdmSlave   *slave,
                                   const char *username,
                                   char      **filenamep)
 {
-        XServerInterpretedAddress si_entries[2];
-        XHostAddress              host_entries[2];
+        XServerInterpretedAddress si_entries[3];
+        XHostAddress              host_entries[3];
         gboolean                  res;
         GError                   *error;
         char                     *filename;
@@ -1467,15 +1474,6 @@ activate_session_id_for_systemd (GdmSlave   *slave,
         GError *error = NULL;
         GVariant *reply;
 
-        /* Can't activate what's already active. We want this
-         * to fail, because we don't want migration to succeed
-         * if the only active session is the one just created
-         * at the login screen.
-         */
-        if (sd_session_is_active (session_id) > 0) {
-                return FALSE;
-        }
-
         reply = g_dbus_connection_call_sync (slave->priv->connection,
                                              "org.freedesktop.login1",
                                              "/org/freedesktop/login1",
@@ -1552,6 +1550,59 @@ activate_session_id (GdmSlave   *slave,
 #endif
 }
 
+#ifdef WITH_CONSOLE_KIT
+static gboolean
+ck_session_is_active (GdmSlave   *slave,
+                      const char *seat_id,
+                      const char *session_id)
+{
+        GError *error = NULL;
+        GVariant *reply;
+        gboolean is_active;
+
+        reply = g_dbus_connection_call_sync (slave->priv->connection,
+                                             CK_NAME,
+                                             session_id,
+                                             "org.freedesktop.ConsoleKit.Session",
+                                             "IsActive",
+                                             NULL,
+                                             G_VARIANT_TYPE ("(b)"),
+                                             G_DBUS_CALL_FLAGS_NONE,
+                                             -1,
+                                             NULL,
+                                             &error);
+        if (reply == NULL) {
+                g_debug ("GdmSlave: ConsoleKit IsActive %s raised:\n %s\n\n",
+                         g_dbus_error_get_remote_error (error), error->message);
+                g_error_free (error);
+                return FALSE;
+        }
+
+        g_variant_get (reply, "(b)", &is_active);
+        g_variant_unref (reply);
+
+        return is_active;
+}
+#endif
+
+static gboolean
+session_is_active (GdmSlave   *slave,
+                   const char *seat_id,
+                   const char *session_id)
+{
+#ifdef WITH_SYSTEMD
+        if (sd_booted () > 0) {
+                return sd_session_is_active (session_id) > 0;
+        }
+#endif
+
+#ifdef WITH_CONSOLE_KIT
+        return ck_session_is_active (slave, seat_id, session_id);
+#else
+        return FALSE;
+#endif
+}
+
 #ifdef WITH_SYSTEMD
 static gboolean
 session_unlock_for_systemd (GdmSlave   *slave,
@@ -1638,10 +1689,12 @@ session_unlock (GdmSlave   *slave,
 
 gboolean
 gdm_slave_switch_to_user_session (GdmSlave   *slave,
-                                  const char *username)
+                                  const char *username,
+                                  gboolean    fail_if_already_switched)
 {
         gboolean    res;
         gboolean    ret;
+        gboolean    session_already_switched;
         char       *ssid_to_activate;
 
         ret = FALSE;
@@ -1652,14 +1705,23 @@ gdm_slave_switch_to_user_session (GdmSlave   *slave,
                 goto out;
         }
 
+        session_already_switched = session_is_active (slave, slave->priv->display_seat_id, ssid_to_activate);
+
         g_debug ("GdmSlave: Activating session: '%s'", ssid_to_activate);
 
-        res = activate_session_id (slave, slave->priv->display_seat_id, ssid_to_activate);
-        if (! res) {
-                g_debug ("GdmSlave: unable to activate session: %s", ssid_to_activate);
+        if (session_already_switched && fail_if_already_switched) {
+                g_debug ("GdmSlave: unable to activate session since it's already active: %s", ssid_to_activate);
                 goto out;
         }
 
+        if (!session_already_switched) {
+                res = activate_session_id (slave, slave->priv->display_seat_id, ssid_to_activate);
+                if (! res) {
+                        g_debug ("GdmSlave: unable to activate session: %s", ssid_to_activate);
+                        goto out;
+                }
+        }
+
         res = session_unlock (slave, ssid_to_activate);
         if (!res) {
                 /* this isn't fatal */
