@@ -366,7 +366,7 @@ signal_cb (int      signo,
                 g_object_unref (settings);
                 settings = gdm_settings_new ();
                 if (settings != NULL) {
-                        if (! gdm_settings_direct_init (settings, GDMCONFDIR "/gdm.schemas", "/")) {
+                        if (! gdm_settings_direct_init (settings, DATADIR "/gdm/gdm.schemas", "/")) {
                                 g_warning ("Unable to initialize settings");
                         }
                 }
