@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
 
@@ -61,14 +61,14 @@ static char      **args_remaining   = NULL;
 
 /* Keep all config options for compatibility even if they are noops */
 GOptionEntry options [] = {
-        { "command", 'c', 0, G_OPTION_ARG_STRING, &send_command, N_("Only the VERSION command is supported"), N_("COMMAND") },
-        { "xnest", 'n', 0, G_OPTION_ARG_NONE, &use_xnest, N_("Ignored — retained for compatibility"), NULL },
-        { "no-lock", 'l', 0, G_OPTION_ARG_NONE, &no_lock, N_("Ignored — retained for compatibility"), NULL },
-        { "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_in, N_("Debugging output"), NULL },
-        { "authenticate", 'a', 0, G_OPTION_ARG_NONE, &authenticate, N_("Ignored — retained for compatibility"), NULL },
-        { "startnew", 's', 0, G_OPTION_ARG_NONE, &startnew, N_("Ignored — retained for compatibility"), NULL },
+        { "command", 'c', 0, G_OPTION_ARG_STRING, &send_command, "Only the VERSION command is supported", "COMMAND" },
+        { "xnest", 'n', 0, G_OPTION_ARG_NONE, &use_xnest, "Ignored — retained for compatibility", NULL },
+        { "no-lock", 'l', 0, G_OPTION_ARG_NONE, &no_lock, "Ignored — retained for compatibility", NULL },
+        { "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_in, "Debugging output", NULL },
+        { "authenticate", 'a', 0, G_OPTION_ARG_NONE, &authenticate, "Ignored — retained for compatibility", NULL },
+        { "startnew", 's', 0, G_OPTION_ARG_NONE, &startnew, "Ignored — retained for compatibility", NULL },
         { "monte-carlo-pi", 0, 0, G_OPTION_ARG_NONE, &monte_carlo_pi, NULL, NULL },
-        { "version", 0, 0, G_OPTION_ARG_NONE, &show_version, N_("Version of this application"), NULL },
+        { "version", 0, 0, G_OPTION_ARG_NONE, &show_version, "Version of this application", NULL },
         { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &args_remaining, NULL, NULL },
         { NULL }
 };
@@ -354,7 +354,7 @@ seat_can_activate_sessions (GDBusConnection *connection,
                 return FALSE;
         }
 
-        g_variant_get (reply, "(&b)", &ret);
+        g_variant_get (reply, "(b)", &ret);
         g_variant_unref (reply);
 
         return ret;
@@ -443,14 +443,14 @@ goto_login_session_for_ck (GDBusConnection  *connection,
         seat_id = get_current_seat_id (connection);
         if (seat_id == NULL || seat_id[0] == '\0') {
                 g_debug ("seat id is not set; can't switch sessions");
-                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, _("Could not identify the current session."));
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "Could not identify the current session.");
 
                 return FALSE;
         }
 
         res = get_login_window_session_id_for_ck (connection, seat_id, &session_id);
         if (! res) {
-                g_set_error (error, GDM_FLEXISERVER_ERROR, 1, _("User unable to switch sessions."));
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 1, "User unable to switch sessions.");
                 return FALSE;
         }
 
@@ -511,6 +511,8 @@ get_login_window_session_id_for_systemd (const char  *seat_id,
         int        res, i;
         char     **sessions;
         char      *service_id;
+        char      *service_class;
+        char      *state;
 
         res = sd_seat_get_sessions (seat_id, &sessions, NULL, NULL);
         if (res < 0) {
@@ -526,6 +528,33 @@ get_login_window_session_id_for_systemd (const char  *seat_id,
 
         for (i = 0; sessions[i]; i ++) {
 
+                res = sd_session_get_class (sessions[i], &service_class);
+                if (res < 0) {
+                        g_debug ("failed to determine class of session %s: %s", sessions[i], strerror (-res));
+                        ret = FALSE;
+                        goto out;
+                }
+
+                if (strcmp (service_class, "greeter") != 0) {
+                        free (service_class);
+                        continue;
+                }
+
+                free (service_class);
+
+                ret = sd_session_get_state (sessions[i], &state);
+                if (ret < 0) {
+                        g_debug ("failed to determine state of session %s: %s", sessions[i], strerror (-res));
+                        ret = FALSE;
+                        goto out;
+                }
+
+                if (g_strcmp0 (state, "closing") == 0) {
+                        free (state);
+                        continue;
+                }
+                free (state);
+
                 res = sd_session_get_service (sessions[i], &service_id);
                 if (res < 0) {
                         g_debug ("failed to determine service of session %s: %s", sessions[i], strerror (-res));
@@ -581,7 +610,7 @@ goto_login_session_for_systemd (GDBusConnection  *connection,
         res = sd_pid_get_session (0, &our_session);
         if (res < 0) {
                 g_debug ("failed to determine own session: %s", strerror (-res));
-                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, _("Could not identify the current session."));
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "Could not identify the current session.");
 
                 return FALSE;
         }
@@ -590,7 +619,7 @@ goto_login_session_for_systemd (GDBusConnection  *connection,
         free (our_session);
         if (res < 0) {
                 g_debug ("failed to determine own seat: %s", strerror (-res));
-                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, _("Could not identify the current seat."));
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "Could not identify the current seat.");
 
                 return FALSE;
         }
@@ -600,7 +629,7 @@ goto_login_session_for_systemd (GDBusConnection  *connection,
                 free (seat_id);
 
                 g_debug ("failed to determine whether seat can do multi session: %s", strerror (-res));
-                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, _("The system is unable to determine whether to switch to an existing login screen or start up a new login screen."));
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "The system is unable to determine whether to switch to an existing login screen or start up a new login screen.");
 
                 return FALSE;
         }
@@ -608,7 +637,7 @@ goto_login_session_for_systemd (GDBusConnection  *connection,
         if (res == 0) {
                 free (seat_id);
 
-                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, _("The system is unable to start up a new login screen."));
+                g_set_error (error, GDM_FLEXISERVER_ERROR, 0, "The system is unable to start up a new login screen.");
 
                 return FALSE;
         }
@@ -674,7 +703,7 @@ main (int argc, char *argv[])
         setlocale (LC_ALL, "");
 
         /* Option parsing */
-        ctx = g_option_context_new (_("- New GDM login"));
+        ctx = g_option_context_new ("- New GDM login");
         g_option_context_set_translation_domain (ctx, GETTEXT_PACKAGE);
         g_option_context_add_main_entries (ctx, options, NULL);
         g_option_context_parse (ctx, &argc, &argv, NULL);
@@ -730,7 +759,7 @@ main (int argc, char *argv[])
                                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                                  GTK_MESSAGE_ERROR,
                                                  GTK_BUTTONS_CLOSE,
-                                                 "%s", _("Unable to start new display"));
+                                                 "%s", "Unable to start new display");
 
                 gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
                                                           "%s", message);
